/*
   Tables.c : Data tables for the Linux Trace Toolkit.
   Copyright (C) 1999, 2000 Karim Yaghmour (karym@opersys.com).
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   History : 
     K.Y., 31/05/2000, Added generalized support for multiple tables
     K.Y., 29/10/99, Initial typing.
*/

#include "Tables.h"
#include "LinuxEvents.h"
#include "LinuxTables.h"
#if SUPP_RTAI
#include "RTAIEvents.h"
#include "RTAITables.h"
#endif /* SUPP_RTAI */
#include "EventDB.h"

/* Initial values of structures */
int**   EventStructSize = (int**)  &LinuxEventStructSize;

#if 0
char**  EventID         = (char**)  LinuxEventID;
#else
char*   (*EventString)(db*, int, event*) = &LinuxEventString;
#endif

#if 0
char**  SyscallID       = (char**)  LinuxSyscallID;
char**  TrapID          = (char**)  LinuxTrapID;
#else
char**  SyscallID;
char**  TrapID;
#endif

/* We don't know the type of trace before hand. Therefore, use the table that includes
   the most events possible. */
#if SUPP_RTAI
int     MaxEventID      = TRACE_RTAI_MAX;
char**  EventOT         = (char**)  RTAIEventOT;
#else
int     MaxEventID      = TRACE_MAX;
char**  EventOT         = (char**)  LinuxEventOT;
#endif

/******************************************************************
 * Function :
 *    TSetTraceType()
 * Description :
 *    Sets the type of trace being analyzed so that the event related
 *    tables match the type of trace analyzed.
 * Parameters :
 *    pmArchType, Type of architecture.
 *    pmSystemType, System type.
 * Return values :
 *    NONE
 * History :
 *    K.Y., 31/05/2000, Initial typing.
 * Note :
 ******************************************************************/
void TSetTraceType(int pmArchType, int pmSystemType)
{
  /* Depending on the type of system */
  switch(pmSystemType)
    {
    case TRACE_SYS_TYPE_VANILLA_LINUX:
#if 0
      printf("TraceToolkit: Trace is vanilla linux kernel \n");
#endif
      /* Initialize value common to all architectures running linux */
      MaxEventID      = TRACE_MAX;
      EventStructSize = (int**) &LinuxEventStructSize;
#if 0
      EventID         = (char**) LinuxEventID;
#else
      EventString     = &LinuxEventString;
#endif
      EventOT         = (char**) LinuxEventOT;

      /* Depending on the architecture */
      switch(pmArchType)
	{
	case TRACE_ARCH_TYPE_I386:
	  /* Set everything to reflect Linux's tables on a i386 */
	  SyscallID       = (char**) LinuxI386SyscallID;
	  TrapID          = (char**) LinuxI386TrapID;
	  break;

	case TRACE_ARCH_TYPE_PPC:
	  /* Set everything to reflect Linux's tables on a PPC */
	  SyscallID       = (char**) LinuxPPCSyscallID;
	  TrapID          = (char**) LinuxPPCTrapID;
	  break;
	}
      break;

#if SUPP_RTAI
    case TRACE_SYS_TYPE_RTAI_LINUX:
#if 0
      printf("TraceToolkit: Trace contains RTAI data \n");
#endif

      /* Use RTAI values */
      MaxEventID      = TRACE_RTAI_MAX;
      EventStructSize = (int**) &RTAIEventStructSize;
#if 0
      EventID         = (char**) RTAIEventID;
#else
      EventString     = &RTAIEventString;
#endif
      EventOT         = (char**) RTAIEventOT;

      /* The syscall IDs and trap IDs remain Linux' */
      SyscallID       = (char**) LinuxI386SyscallID;
      TrapID          = (char**) LinuxI386TrapID;
      break;
#endif /* SUPP_RTAI */

    default:
#if 1
      printf("TraceToolkit: Unknown system type %d \n", pmSystemType);
#endif
      exit(1);
      break;
    }
}
